package es.logmeal.sdk;

import java.util.ArrayList;

public class AngleComputation {
    private float[] cam0Proj;
    private float[] center;
    private ArrayList<float[][]> arcoreCamsMatrix;

    public AngleComputation(float[] cam0Proj, float[] center,ArrayList<float[][]> arcoreCamsMatrix){
        this.cam0Proj = cam0Proj;
        this.center = center;
        this.arcoreCamsMatrix = arcoreCamsMatrix;
    }


    public  double[] getAngleWithRespectToFirstCam() {
        double[] relativeTheta = new double[arcoreCamsMatrix.size()];

        // The third column of the first camera matrix defines the camera position of the first camera
        // We have to obtain the position of the first camera with respect to the scene center (using a subtraction)
        float[] cam2CenterVector0 = subtractVectors(cam0Proj, center);

        // Normalize the vector dividing it by its norm
        cam2CenterVector0 = normalizeVector(cam2CenterVector0);

        for (int i = 0; i < arcoreCamsMatrix.size(); i++) {
            float[][] cam = arcoreCamsMatrix.get(i);

            // The third column of each camera matrix defines the camera position
            // We have to obtain the position of the camera with respect to the scene center (using a subtraction)
            float[] cam2CenterVector_aux = getColumn(cam, 3);


            float[] cam2CenterVector_aux2 = new float[]{cam2CenterVector_aux[0], cam2CenterVector_aux[1], cam2CenterVector_aux[2]};

            float[] cam2CenterVector = subtractVectors(cam2CenterVector_aux2, center);

            // Normalize the vector dividing it by its norm
            cam2CenterVector = normalizeVector(cam2CenterVector);

            // Obtain the angle between the first and current camera
            double theta = Math.toDegrees(Math.acos(dotProduct(cam2CenterVector0, cam2CenterVector)));
            relativeTheta[i] = theta;
        }

        return relativeTheta;
    }
    private static float[] getColumn(float[][] matrix, int columnIndex) {
        float[] column = new float[matrix.length];
        for (int i = 0; i < matrix.length; i++) {
            column[i] = matrix[i][columnIndex];
        }
        return column;
    }
    private static double dotProduct(float[] vector1, float[] vector2) {
        double result = 0.0;
        for (int i = 0; i < vector1.length; i++) {
            result += vector1[i] * vector2[i];
        }
        return result;
    }
    private static float[] subtractVectors(float[] vector1, float[] vector2) {
        float[] result = new float[vector1.length];
        for (int i = 0; i < vector1.length; i++) {
            result[i] = vector1[i] - (float) vector2[i];
        }
        return result;
    }
    private static float[] normalizeVector(float[] vector) {
        double norm = 0.0;
        for (float v : vector) {
            norm += v * v;
        }
        norm = Math.sqrt(norm);

        float[] normalizedVector = new float[vector.length];
        for (int i = 0; i < vector.length; i++) {
            normalizedVector[i] = vector[i] / (float) norm;
        }
        return normalizedVector;
    }
}
