package es.logmeal.sdk;

import android.content.Context;
import android.content.res.Configuration;
import android.graphics.ImageFormat;
import android.media.ExifInterface;
import android.media.Image;

import com.google.ar.core.Frame;
import com.google.ar.core.exceptions.NotYetAvailableException;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.nio.ByteBuffer;
import java.nio.channels.FileChannel;
import java.nio.file.Path;

import es.logmeal.sdk.CameraInfo;
import es.logmeal.sdk.exceptions.IncompatibleRgbImageType;
import es.logmeal.sdk.helpers.ImageConvertionUtils;

public class FrameInfo {
    private Path multipleOutPath;

    private Context context;

    public FrameInfo(Path multipleOutPath, Context context){
        this.multipleOutPath = multipleOutPath;
        this.context = context;
    }

     public CameraInfo getCameraFromFrame(Frame frame, float[] poseMatrix) {

        float[] focalLength = frame.getCamera().getImageIntrinsics().getFocalLength();
        float[] principalPoint = frame.getCamera().getImageIntrinsics().getPrincipalPoint();
        return new CameraInfo(focalLength, principalPoint, poseMatrix);
    }
    public File prepareOutputFile(String prefix, String suffix) throws IOException {
        String outputPath = multipleOutPath.toString();
        long identifier = System.currentTimeMillis();
        return outputPath == null ? File.createTempFile("image" + prefix, suffix) :
                new File(outputPath, "image" + prefix + suffix);
    }
    public void getImageData(Frame frame, int numFramesTaken) throws NotYetAvailableException, IncompatibleRgbImageType, IOException {
        Image originalImage = frame.acquireCameraImage();


        int format = originalImage.getFormat();
        if (format != ImageFormat.YUV_420_888) {
            throw new IncompatibleRgbImageType();
        }

        byte[] jpegImage = ImageConvertionUtils.toJpegImageBytes(originalImage, 100);
        File originalImageFile = prepareOutputFile(String.valueOf(numFramesTaken), ".jpeg");
        FileChannel fc = new FileOutputStream(originalImageFile).getChannel();
        fc.write(ByteBuffer.wrap(jpegImage));
        fc.close();
        originalImage.close();

        // cropping dimensions


        int orientation = context.getResources().getConfiguration().orientation;
        // no cropping

        if (orientation == Configuration.ORIENTATION_PORTRAIT) {
            ExifInterface exifInterface = new ExifInterface(originalImageFile.getPath());
            exifInterface.setAttribute(ExifInterface.TAG_ORIENTATION, String.valueOf(ExifInterface.ORIENTATION_ROTATE_90));
            exifInterface.saveAttributes();
        }
        //return new ProcessedRgbImg(originalImageFile, originalImageWidth, originalImageHeight, 0, 0, orientation, null, originalImageWidth, originalImageHeight, multipleOutPath.toString());



    }
}
