/*
* This class is created to manage the input for the API of LogMeal
* */
package es.logmeal.sdk;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.nio.file.Path;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import es.logmeal.sdk.JCodecImageToVideoConverter;
import es.logmeal.sdk.MyTuple;
public class GenerateInputAPI {
    private Path multipleOutPath;
    private MyTuple[] framePerAngle;
    private List<Integer> indexs;
    private int closestIndex;
    private int closestIndex_after_video;

    private String cam_focal_length = "";
    private String cam_principal_point = "";
    private String main_rgb_idx = "";
    private String camera_pose = "";


    public GenerateInputAPI(Path multipleOutPath, MyTuple[] framePerAngle) {
        this.multipleOutPath = multipleOutPath;
        this.framePerAngle = framePerAngle;
        closestIndex = getIndexOfMainRGBImage();
    }

    // Method used to generate the video sequence and the JSON
    public void generateInputsForAPI() throws IOException {
        generateVideoInputForAPI();
        generateJSONInputForAPI();
    }

    // Method that saves all the camera poses into a List
    public List<float[]> generateDataPose() {
        List<float[]> dataPose = new ArrayList<>();
        for (int i = 0; i < framePerAngle.length; i++) {
            dataPose.add(framePerAngle[i].getCameraPose());
        }
        return dataPose;
    }

    // Method to obtain the index of the main RGB image. The RGB image is the closest image taken
    // to the angle 90.
    public int getIndexOfMainRGBImage() {
        double target = 90;
        double[] values = new double[framePerAngle.length];
        for (int i = 0; i < framePerAngle.length; i++) {
            values[i] = framePerAngle[i].getAngle();
        }
        double closestValue = values[0];  // Initialize with the first element
        closestIndex = 0;
        double minDifference = Math.abs(values[0] - target);  // Initialize with the difference
        // of the first element

        // Iterate through the array
        for (int i = 1; i < values.length; i++) {
            double difference = Math.abs(values[i] - target);

            // Check if the current element is closer to the target
            if (difference < minDifference) {
                closestValue = values[i];
                closestIndex = i;
                minDifference = difference;
            }
        }
        System.out.println("Closest value: " + closestValue);
        System.out.println("Index of closest value: " + closestIndex);
        return closestIndex;
    }

    // Method to generate the video
    public void generateVideoInputForAPI() {
        JCodecImageToVideoConverter jCodecImageToVideoConverter =
                new JCodecImageToVideoConverter(multipleOutPath.toString(), closestIndex);
        jCodecImageToVideoConverter.createVideo();
        indexs = jCodecImageToVideoConverter.getIdxs();
        closestIndex_after_video = jCodecImageToVideoConverter.getClosestIndex_after_video();
    }

    // Method to generate the JSON
    public void generateJSONInputForAPI() throws IOException {
        List<float[]> dataPose = generateDataPose();
        System.out.println("List of indexes of images: ");
        for (Integer value : indexs)
            System.out.println(value);
        writeFinalJSON(closestIndex_after_video, framePerAngle[0].getPrincipalPoint(),
                framePerAngle[0].getFocalLength(), multipleOutPath, dataPose, indexs);
    }

    // Method that writes in memory the JSON. The JSON data needed are: all the camera poses of the
    // frames taken, the focal length and principal point of the camera, the index of the main
    // RGB image and the version of the operating system
    private void writeFinalJSON(int indexMainImage, float[] principalPoint, float[] focalLength,
                                Path path, List<float[]> cameraPoses, List<Integer> indexs) throws IOException {

        String info = "";
        info += "{" ;
        info += "\"cam_focal_length\": {\'x\':" + focalLength[0]/(1.33333333333) + ",\'y\':" + focalLength[1]/(1.33333333333) + "}," + "\n";
        cam_focal_length = "{\'x\':" + focalLength[0]/(1.33333333333) + ",\'y\':" + focalLength[1]/(1.33333333333) + "}";
        info += "\"cam_principal_point\": {\'x\':" + principalPoint[0]/(1.33333333333) + ",\'y\': " + principalPoint[1]/(1.33333333333) + "}," + "\n";
        cam_principal_point =  "{\'x\':" + principalPoint[0]/(1.33333333333) + ",\'y\':" + principalPoint[1]/(1.33333333333) + "}";
        info += "\"main_rgb_idx\": " + indexMainImage + ",\n";
        main_rgb_idx = String.valueOf(indexMainImage);
        info += "\"camera_pose\": {";
        for (int i = 0; i < indexs.size(); i++) {
            String formattedIndex = "\'" + String.format("%d", i) + "\'";
            if (i != indexs.size() - 1)
                info += formattedIndex + ":" + Arrays.toString(cameraPoses.get(i)) + ",";
            else
                info += formattedIndex + ":" + Arrays.toString(cameraPoses.get(i));
        }
        info += "}," + "\n";
        camera_pose = "{";
        for (int i = 0; i < indexs.size(); i++) {
            String formattedIndex = "\'" + String.format("%d", i) + "\'";
            if (i != indexs.size() - 1)
                camera_pose += formattedIndex + ":" + Arrays.toString(cameraPoses.get(i)) + ",";
            else
                camera_pose += formattedIndex + ":" + Arrays.toString(cameraPoses.get(i));
        }
        camera_pose += "}";
        info += "\"version\": \"android\"\n" + "}";

        File infoFile = new File(multipleOutPath.toString(),
                path.getFileName().toString() + "-" + "json.txt");
        FileOutputStream stream = new FileOutputStream(infoFile);
        stream.write(info.getBytes());
        stream.close();
    }

    public int getClosestIndex() {
        return closestIndex;
    }
    public String getCam_focal_length(){
        return cam_focal_length;
    }
    public String getCam_principal_point(){
        return cam_principal_point;
    }
    public String getMain_rgb_idx(){
        return main_rgb_idx;
    }
    public String getCamera_pose(){
        return camera_pose;
    }
}
