package es.logmeal.sdk;

import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.os.Environment;
import android.util.Log;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;

public class ImageProcessor {

    public static void resizeAndSaveImage(String originalFilePath, String newFilePath, int targetWidth, int targetHeight) {
        // Load the original image
        Bitmap originalBitmap = BitmapFactory.decodeFile(originalFilePath);

        // Resize the image
        Bitmap resizedBitmap = Bitmap.createScaledBitmap(originalBitmap, targetWidth, targetHeight, true);

        // Save the resized image
        saveBitmap(resizedBitmap, newFilePath);

        // Delete the original image
        deleteFile(originalFilePath);

        // Recycle the bitmaps to free up memory
        originalBitmap.recycle();
        resizedBitmap.recycle();
    }

    private static void saveBitmap(Bitmap bitmap, String filePath) {
        try (FileOutputStream out = new FileOutputStream(filePath)) {
            bitmap.compress(Bitmap.CompressFormat.JPEG, 100, out);
        } catch (IOException e) {
            Log.e("ImageProcessor", "Error saving bitmap: " + e.getMessage());
        }
    }

    private static void deleteFile(String filePath) {
        File file = new File(filePath);
        if (file.exists()) {
            boolean deleted = file.delete();
            if (!deleted) {
                Log.e("ImageProcessor", "Error deleting original file");
            }
        }
    }
}
