/*
* This class contains all the resources needed to generate a video using jcodec.
* */
package es.logmeal.sdk;

import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.util.Log;

import org.jcodec.api.android.AndroidSequenceEncoder;
import org.jcodec.common.io.NIOUtils;
import org.jcodec.common.io.SeekableByteChannel;
import org.jcodec.common.model.Rational;

import java.io.File;
import java.io.FilenameFilter;
import java.io.IOException;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.util.ArrayList;
import java.util.List;

public class JCodecImageToVideoConverter {
    private static final String OUTPUT_VIDEO_PATH = "output.mp4";
    private String path;
    private final int goalFrames = 50, closestIndex;

    private int closestIndex_after_video = 0;
    int counter = 0;
    private List<Integer> idxs = new ArrayList<>();

    JCodecImageToVideoConverter(String path, int closestIndex) {
        this.path = path;
        this.closestIndex = closestIndex;
    }

    // Method to create the video
    public void createVideo() {
        try {

            File outputDir = new File(path);
            outputDir.mkdirs();
            int jpegFileCount = 0;
            if (outputDir.exists() && outputDir.isDirectory()) {
                // Count the number of .jpeg files in the directory
                jpegFileCount = countJPEGFiles(outputDir);
                // Print the result
                System.out.println("Number of .jpeg files: " + jpegFileCount);
            } else {
                System.out.println("The specified directory does not exist or is not a directory.");
            }
            File outputFile = new File(outputDir, "output.avi");

            SeekableByteChannel out = NIOUtils.writableFileChannel(outputFile.getAbsolutePath());
            // Variables needed for the video resolution
            int width = 480;
            int height = 360;
            // Create a SequenceEncoder with the desired output video file
            AndroidSequenceEncoder sequenceEncoder = new AndroidSequenceEncoder(out,
                    Rational.R(60, 1));

            boolean discardImages = (jpegFileCount > goalFrames) ? true : false;
            int difference = (jpegFileCount > goalFrames) ? (jpegFileCount - goalFrames) : 0;

            for (int i = 0; i < jpegFileCount; i++) {
                // Condition 1: Check if it's the closest odd index to be included
                if (i == closestIndex && i % 2 != 0) {
                    encodeImage(i, path, sequenceEncoder, width, height);

                }
                // Condition 2: Skip odd indices based on the difference (if needed)
                else if (i % 2 != 0 && difference != 0 && discardImages) {
                    difference--;
                    // Condition 3: Include all other images
                } else {
                    // Special case: Skip the next image when it's the closestIndex + 1
                    if (i == closestIndex + 1 && i % 2 == 0 && difference != 0 && discardImages) {
                        difference--;
                    }
                    // Encode the image
                    else {
                        encodeImage(i, path, sequenceEncoder, width, height);
                    }

                }

            }
            // Finalize the video encoding
            sequenceEncoder.finish();

            System.out.println("Video created successfully: " + OUTPUT_VIDEO_PATH);

        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    // Method that encodes an image.
    private void encodeImage(int index, String path, AndroidSequenceEncoder sequenceEncoder,
                             int width, int height) throws IOException {
        idxs.add(index);
        if(index == closestIndex){
            closestIndex_after_video = counter;
            System.out.println("L'index del video es " + closestIndex_after_video);
        }

        counter++;
        byte[] imageBytes = readImageFile(index, path);
        // Add image to the video
        Bitmap bitmap = convertBytesToBitmap(imageBytes);
        // Resize the bitmap to the desired resolution
        Bitmap resizedBitmap = Bitmap.createScaledBitmap(bitmap, width, height, true);
        sequenceEncoder.encodeImage(resizedBitmap);
    }
    public int getClosestIndex_after_video(){return closestIndex_after_video;}

    private Bitmap convertBytesToBitmap(byte[] imageData) {
        if (imageData == null || imageData.length == 0) {
            return null;
        }

        // Decode the byte array into a Bitmap
        return BitmapFactory.decodeByteArray(imageData, 0, imageData.length);
    }

    // Method that read the images files from the folder. The images previously stored follow the
    // next name structure: imageX.jpeg where X is an int.
    private static byte[] readImageFile(int index, String multipleOutPath) {
        String imageName = "image" + index + ".jpeg";  // Change the extension according to your
        // image format
        Path imagePath = Paths.get(multipleOutPath, imageName);
        Log.d("esnull", imagePath.toString());
        try {
            return Files.readAllBytes(imagePath);
        } catch (IOException e) {
            e.printStackTrace();
            return null;
        }
    }
    // Method to count the images that have been stored
    private static int countJPEGFiles(File directory) {
        // Create a FilenameFilter to filter files with .jpeg extension
        FilenameFilter jpegFilter = (dir, name) -> name.toLowerCase().endsWith(".jpeg");

        // List files in the directory that match the filter
        String[] jpegFiles = directory.list(jpegFilter);

        // Return the count of .jpeg files
        return (jpegFiles != null) ? jpegFiles.length : 0;
    }

    public List<Integer> getIdxs() {
        return idxs;
    }

}

