package es.logmeal.sdk;

import org.apache.commons.math3.linear.MatrixUtils;
import org.apache.commons.math3.linear.RealMatrix;

import java.util.ArrayList;


public class SceneCenterComputation {

    private Center center;
    public SceneCenterComputation(){

    }
    public Center initialize_scene_center() {
        // Weight for each camera pair
        float totw = 0.0F;
        float[] totp = {0.0F, 0.0F, 0.0F};
        this.center = new Center(totw, totp);
        return center;
    }
    public Center update_scene_center_from_cams(ArrayList<float[][]> trajectories_2d,float[][] trajectory) {

        float totw = center.getTotw();
        float[] totp = center.getTotp();

        // We select the first 3 rows of the camera


        int numRowsToExtract = 3;
        int numCols = trajectory[0].length;

        float[][] lc = new float[numRowsToExtract][numCols];

        for (int i = 0; i < numRowsToExtract; i++)
            System.arraycopy(trajectory[i], 0, lc[i], 0, numCols);

        int numRows = lc.length;
        float[] lc_col_3 = new float[numRows];
        float[] lc_col_2 = new float[numRows];

        int colIndex_3 = 3; // Column index for lc_col_3
        int colIndex_2 = 2; // Column index for lc_col_2

        for (int i = 0; i < numRows; i++) {
            lc_col_3[i] = lc[i][colIndex_3];
            lc_col_2[i] = lc[i][colIndex_2];
        }


        for (int i = 0; i < trajectories_2d.size(); i++) {
            float[][] tc = new float[numRowsToExtract][numCols];

            for (int j = 0; j < numRowsToExtract; j++) {
                System.arraycopy(trajectories_2d.get(i)[j], 0, tc[j], 0, numCols);
            }

            float[] tc_col_3 = new float[numRows];
            float[] tc_col_2 = new float[numRows];

            for (int k = 0; k < numRows; k++) {
                tc_col_3[k] = tc[k][colIndex_3];
                tc_col_2[k] = tc[k][colIndex_2];

            }


            //closest_point_2_lines
            Object[] closest_point = closest_point_2_lines(lc_col_3, lc_col_2, tc_col_3, tc_col_2);

            float[] p = (float[]) closest_point[0];
            float w_two = (float) closest_point[1];
            if (w_two > 0.00001) {
                for (int n = 0; n < p.length; n++)
                    totp[n] += (p[n] * w_two);
                totw += w_two;
            }
        }
        center = new Center(totw, totp);
        return center;
    }
    public Object[] closest_point_2_lines(float[] oa, float[] da, float[] ob, float[] db) {
        // returns point closest to both rays of form o+t*d

        float norm_da = norm(da);
        float[] aux_da = normalize(da, norm_da);
        float norm_db = norm(db);
        float[] aux_db = normalize(db, norm_db);

        float[] c = calculateCrossProduct(aux_da, aux_db);
        float denom = norm(c) * norm(c);
        float[] t = new float[oa.length];
        for (int i = 0; i < oa.length; i++)
            t[i] = ob[i] - oa[i];
        double[][] aux_ta = new double[oa.length][oa.length];
        double[][] aux_tb = new double[oa.length][oa.length];
        // Populate the 2D array using elements from x, y, and z
        for (int i = 0; i < 3; i++) {
            aux_ta[i][0] = t[i];
            aux_ta[i][1] = db[i];
            aux_ta[i][2] = c[i];

            aux_tb[i][0] = t[i];
            aux_tb[i][1] = da[i];
            aux_tb[i][2] = c[i];
        }
        RealMatrix m_ta = MatrixUtils.createRealMatrix(aux_ta);
        double det_ta = calculateDeterminant(m_ta);
        RealMatrix m_tb = MatrixUtils.createRealMatrix(aux_tb);
        double det_tb = calculateDeterminant(m_tb);
        double ta = det_ta / (denom + 1e-10);
        double tb = det_tb / (denom + 1e-10);
        if (ta > 0) ta = 0;
        if (tb > 0) tb = 0;
        float[] result = new float[oa.length];
        for (int i = 0; i < oa.length; i++)
            result[i] = (float) ((oa[i] + (ta * da[i]) + ob[i] + (tb * db[i])) * 0.5);
        return new Object[]{result, denom};

    }
    public float[] apply_weight_scene_center() {

        float totw = center.getTotw();
        float[] aux_totp = center.getTotp();
        float[] totp = new float[aux_totp.length];
        for (int i = 0; i < totp.length; i++)
            totp[i] = aux_totp[i];
        if (totw > 0.0) for (int i = 0; i < totp.length; i++)
            totp[i] /= totw;
        return totp;
    }
    public static double calculateDeterminant(RealMatrix matrix) {
        return new org.apache.commons.math3.linear.LUDecomposition(matrix).getDeterminant();
    }
    public static float[] calculateCrossProduct(float[] a, float[] b) {

        if (a.length != 3 || b.length != 3)
            throw new IllegalArgumentException("Both input arrays must have exactly 3 elements.");
        float[] result = new float[3];
        result[0] = a[1] * b[2] - a[2] * b[1];
        result[1] = a[2] * b[0] - a[0] * b[2];
        result[2] = a[0] * b[1] - a[1] * b[0];
        return result;
    }
    public float norm(float[] data) {
        double aux = 0.0;
        for (int i = 0; i < data.length; i++)
            aux += (data[i] * data[i]);
        return (float) Math.sqrt(aux);
    }
    public float[] normalize(float[] data, float val) {
        float[] aux = new float[data.length];
        for (int i = 0; i < data.length; i++)
            aux[i] = data[i] / val;
        return aux;
    }

}
