package es.logmeal.sdk;

import android.util.Log;

import java.io.File;
import java.io.IOException;
import java.util.concurrent.Callable;

import okhttp3.MediaType;
import okhttp3.MultipartBody;
import okhttp3.OkHttpClient;
import okhttp3.Request;
import okhttp3.RequestBody;
import okhttp3.Response;

public class SendDepthRequest implements Callable<Void> {

    private final String authToken;
    private final String imagePath;
    private final String sequencePath;
    private final String camFocalLength;
    private final String camPrincipalPoint;
    private final String mainRgbIdx;
    private final String cameraPose;
    private final String version;
    private final String fileName;

    public SendDepthRequest(String authToken, String imagePath, String fileName,String sequencePath,
                            String camFocalLength, String camPrincipalPoint,
                            String mainRgbIdx, String cameraPose, String version) {
        this.authToken = authToken;
        this.imagePath = imagePath;
        this.fileName = fileName;
        this.sequencePath = sequencePath;
        this.camFocalLength = camFocalLength;
        this.camPrincipalPoint = camPrincipalPoint;
        this.mainRgbIdx = mainRgbIdx;
        this.cameraPose = cameraPose;
        this.version = version;
    }

    @Override
    public Void call() {
        try {

            OkHttpClient client = new OkHttpClient.Builder()
                    .readTimeout(30, java.util.concurrent.TimeUnit.SECONDS)
                    .writeTimeout(30, java.util.concurrent.TimeUnit.SECONDS)
                    .build();

            RequestBody requestBody = new MultipartBody.Builder()
                    .setType(MultipartBody.FORM)
                    .addFormDataPart("image", "image.jpeg",
                            RequestBody.create( new File(imagePath),MediaType.parse("image/jpeg")))
                    .addFormDataPart("sequence","output.avi" ,
                            RequestBody.create( new File(sequencePath),MediaType.parse("video/avi")))
                    .addFormDataPart("cam_focal_length", camFocalLength)
                    .addFormDataPart("cam_principal_point", camPrincipalPoint)
                    .addFormDataPart("main_rgb_idx", mainRgbIdx)
                    .addFormDataPart("camera_pose", cameraPose)
                    .addFormDataPart("version", version)
                    .build();
/*
            System.out.println("\nToken:"+authToken);
            System.out.println("\nFilename"+fileName);
            System.out.println("\nImagePath"+imagePath);
            System.out.println("\nSequencePath"+sequencePath);
            System.out.println("\nCamFocalLength"+camFocalLength);
            System.out.println("\nCamPrincipalPoint"+camPrincipalPoint);
            System.out.println("\nMainRGBIndex"+mainRgbIdx);
            System.out.println("\nCameraPose"+cameraPose);
            System.out.println("\nVersion"+version);
            Log.d("CameraPose",cameraPose);
*/
            Request request = new Request.Builder()
                    .url("https://staging.api.logmeal.es/v2/image/segmentation/complete/quantity")
                    .post(requestBody)
                    .header("Authorization", "Bearer " + authToken)
                    .build();


            Response response = client.newCall(request).execute();
            if(!response.isSuccessful()) {
                Log.i("Response code", " " + response.code());
            }
            else{
                Log.i("Response code", response.code() + " ");
                String results = response.body().string();

                Log.i("OkHTTP Results: ", results);

                response.close();
            }

        } catch (IOException e) {
        e.printStackTrace();
        Log.e("MyApp", "Exception during network request: " + e.getMessage());
    }


        return null;
    }
}