package es.logmeal.sdk.sample;

import androidx.appcompat.app.AppCompatActivity;

import android.graphics.Color;
import android.os.Bundle;
import android.os.Environment;
import android.util.Log;
import android.view.View;
import android.widget.TextView;

import java.io.IOException;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.util.concurrent.CompletableFuture;

import es.logmeal.sdk.FrameDepthInfo;
import es.logmeal.sdk.widgets.DepthCamera;

public class DepthCameraWidgetExampleActivity extends AppCompatActivity {
    public static final String TAG = DepthCameraWidgetExampleActivity.class.getSimpleName();

    private Path outPath;
    private DepthCamera dc;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_depth_camera_widget_example);

        dc = findViewById(R.id.depthCamera4);
        getLifecycle().addObserver(findViewById(R.id.depthCamera4));

        outPath = Paths.get(Environment.getExternalStoragePublicDirectory(Environment.DIRECTORY_DOCUMENTS).getAbsolutePath(), "logmeal");
        if (!Files.exists(outPath)) {
            try {
                Files.createDirectory(outPath);
            } catch (IOException e) {
                Log.e(TAG, "Error creating output directory", e);
            }
        }
        dc.setCustomDepthInfoStorePath(outPath.toString());
    }

    public void captureDepthInfo(View view) {
        TextView tv = findViewById(R.id.textView);
        tv.setTextColor(Color.WHITE);
        TextView tvResult = findViewById(R.id.advancedResultTextView);
        tvResult.setTextColor(Color.WHITE);

        // Capture the next frame and retrieve the CompletableFuture
        CompletableFuture<FrameDepthInfo> future = dc.captureNextFrame();

        // Attach a callback to the CompletableFuture
        future.thenAccept(frameDepthInfo -> {
            // Update the UI with the result
            runOnUiThread(() -> {
                String outputPathText = String.format("Depth info stored in: %s", outPath.toString());
                String resultText = frameDepthInfo.toPrettyJson();
                tv.setText(outputPathText);
                tvResult.setText(resultText);
            });
        });

        future.exceptionally(throwable -> {
            // Handle exceptions here
            Log.e("CameraActivity", "Error capturing depth info", throwable);
            return null;
        });
    }
}