package es.logmeal.sdk.sample;

import androidx.appcompat.app.AppCompatActivity;
import androidx.core.content.ContextCompat;

import android.content.Intent;
import android.os.Bundle;
import android.view.View;
import android.widget.Button;
import android.widget.TextView;
import android.widget.Toast;



import es.logmeal.sdk.helpers.ArCoreSupport;
import es.logmeal.sdk.helpers.ArCoreUtils;
import es.logmeal.sdk.helpers.CameraPermissionHelper;
import es.logmeal.sdk.helpers.PermissionsHelper;

public class MainActivity extends AppCompatActivity {
    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_main);


    }

    public void openWidgetExample(View view){
        Intent intent = new Intent(this, DepthCameraWidgetExampleActivity.class);
        startActivity(intent);
    }

    public void openAdvancedExample(View view){
        Intent intent = new Intent(this, AdvancedIntegrationExampleActivity.class);
        startActivity(intent);
    }

    @Override
    protected void onResume() {
        super.onResume();

        // ARCore requires camera permission to operate.
        if (!CameraPermissionHelper.hasCameraPermission(this)) {
            CameraPermissionHelper.requestCameraPermission(this);
        }

        checkDeviceSupport();
    }

    void checkDeviceSupport() {
        ArCoreSupport arCoreSupport = ArCoreUtils.isDeviceSupported(this);

        TextView view = findViewById(R.id.textViewDepthSupport);
        Button installApkButton = findViewById(R.id.button_install_arcore);

        if (arCoreSupport.isArCoreSupported()) {
            Button openWidgetExampleBtn = findViewById(R.id.button_open_widget_example);
            Button openAdvancedExampleBtn = findViewById(R.id.button_open_advanced_example);

            if (arCoreSupport.isDepthSupported()){
                view.setText(getString(R.string.depth_supported_text));
                view.setTextColor(ContextCompat.getColor(getApplicationContext(), R.color.YellowGreen));
            }
            else{
                view.setText(getString(R.string.depth_not_supported_text));
                view.setTextColor(ContextCompat.getColor(getApplicationContext(), R.color.MediumVioletRed));
            }

            installApkButton.setVisibility(View.INVISIBLE);
            openWidgetExampleBtn.setVisibility(View.VISIBLE);
            openAdvancedExampleBtn.setVisibility(View.VISIBLE);
        }
        else if(arCoreSupport.getArCoreAvailability().isSupported() || arCoreSupport.getArCoreAvailability().isUnknown()){
            view.setText(getString(R.string.arcore_support_unknown));
            view.setTextColor(ContextCompat.getColor(getApplicationContext(), R.color.AliceBlue));
            installApkButton.setVisibility(View.VISIBLE);
        }
        else {
            view.setText(getString(R.string.arcore_not_supported));
            view.setTextColor(ContextCompat.getColor(getApplicationContext(), R.color.MediumVioletRed));
        }
    }

    @Override
    public void onRequestPermissionsResult(int requestCode, String[] permissions, int[] results) {
        super.onRequestPermissionsResult(requestCode, permissions, results);
        if (!CameraPermissionHelper.hasCameraPermission(this)) {
            Toast.makeText(this, "Camera permission is needed to run this application", Toast.LENGTH_LONG)
                    .show();
            if (!PermissionsHelper.shouldShowRequestPermissionRationale(this)) {
                PermissionsHelper.launchPermissionSettings(this);
            }
            finish();
        }
    }
}